import numpy as np

def merge_data(data, metric):
    merged_outcomes = {}
    for key in data.keys():
        results = data[key]
        if metric == 1:
            merged_outcomes[key] = list(map(np.mean, zip(*results)))
        if metric == 2:
            merged_outcomes[key] = list(map(min, zip(*results)))
        if metric == 3:
            merged_outcomes[key] = list(map(max, zip(*results)))
    return merged_outcomes



import PySimpleGUI as sg


def header(text):
    return sg.Text(text, font=('', 30))


def title(text):
    return sg.Text(text, font=('', 20), auto_size_text=True)


def explanation(text):
    return sg.Text(text, font=('', 10), pad=(0, 0))


def configuration_parameter_text(text):
    return sg.Text(text, size=(15, 1), pad=(0, 0))


def configuration_value_text(text, key):
    return sg.Text(text, key=key, auto_size_text=False, pad=(0, 0))


def configuration_horizontal_line():
    return sg.Text('---------------------------------------------------------------------------', pad=(0, 0))


def question_mark_button(key, padding=None):
    return sg.Button(' ? ', font=('', 10, 'bold'), key=key, pad=padding)
    #return sg.Button(key=key, pad=padding, image_filename='..\\src\\gui\\images\\q_mark1.png')


def metric3_radio_buttons(prefix):
    return [sg.Radio('Average', f'{prefix}_radiogroup', key=f'{prefix}_metric_1', enable_events=True),
            sg.Radio('Minimum', f'{prefix}_radiogroup', key=f'{prefix}_metric_2', enable_events=True),
            sg.Radio('Maximum', f'{prefix}_radiogroup', key=f'{prefix}_metric_3', enable_events=True)]


def metric4_radio_buttons(prefix):
    return [sg.Radio('Average', f'{prefix}_radiogroup', key=f'{prefix}_metric_1'),
            sg.Radio('Minimum', f'{prefix}_radiogroup', key=f'{prefix}_metric_2'),
            sg.Radio('Maximum', f'{prefix}_radiogroup', key=f'{prefix}_metric_3'),
            sg.Radio('Last', f'{prefix}_radiogroup', key=f'{prefix}_metric_4')]



import PySimpleGUI as sg
from NetLogoDOE.src.gui.custom_components import title, question_mark_button
from NetLogoDOE.src.gui.help_dictionary import help_text
from NetLogoDOE.src.gui.custom_windows import show_help_window


class ExperimentResultsScreen:

    def __init__(self):
        button_size = (30, 1)
        button_pad = ((5, 0), (20, 0))
        self.layout = [[title("Experiment results")],
                       [sg.Button('Experiment Configuration Information', key='experiment_results_configtable_button',
                                  size=button_size, pad=button_pad),
                        question_mark_button('experiment_results_configtable_help_button', padding=button_pad)],
                       [sg.Button('Parallel Coordinates', key='experiment_results_parcoords_button',
                                  size=button_size, pad=button_pad),
                        question_mark_button('experiment_results_parcoords_help_button', padding=button_pad)],
                       [sg.Button('Scatterplot', key='experiment_results_scatterplot_button',
                                  size=button_size, pad=button_pad),
                        question_mark_button('experiment_results_scatterplot_help_button', padding=button_pad)],
                       [sg.Button('Heatmap', key='experiment_results_heatmap_button',
                                  size=button_size, pad=button_pad),
                        question_mark_button('experiment_results_heatmap_help_button', padding=button_pad)],
                       [sg.Input(key='experiment_results_dummy_export', enable_events=True, visible=False, size=(0, 0)),
                        sg.SaveAs('Save Results', file_types=[("Text Files", "*.txt")],
                                  target='experiment_results_dummy_export', key="experiment_results_save_button",
                                  size=button_size, pad=button_pad),
                        question_mark_button('experiment_results_save_help_button', padding=button_pad)],
                       [sg.Button('Back to main menu', key='experiment_results_back_button',
                                  size=button_size, pad=button_pad)]
                       ]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'experiment_write_results_event':
            self.results = values['experiment_write_results_event']

        if event == 'experiment_results_configtable_button':
            window['experiment_result_panel'].update(visible=False)
            window['experiment_configtable_panel'].update(visible=True)
        if event == 'experiment_results_parcoords_button':
            window['experiment_result_panel'].update(visible=False)
            window['parcoords_panel'].update(visible=True)
        if event == 'experiment_results_scatterplot_button':
            window['experiment_result_panel'].update(visible=False)
            window['scatterplot_panel'].update(visible=True)
        if event == 'experiment_results_heatmap_button':
            window['experiment_result_panel'].update(visible=False)
            window['heatmap_panel'].update(visible=True)

        if event == 'experiment_results_dummy_export' and not (values['experiment_results_dummy_export'] == ''):
            self.export_experiment_results(values, values['experiment_results_dummy_export'])
        if event == 'experiment_results_back_button':
            window['experiment_result_panel'].update(visible=False)
            window['main_panel'].update(visible=True)

        # Help events
        if event == 'experiment_results_configtable_help_button':
            show_help_window(help_text['config_information'], location=window.CurrentLocation())
        if event == 'experiment_results_parcoords_help_button':
            show_help_window(help_text['parallel_coordinates'], location=window.CurrentLocation())
        if event == 'experiment_results_scatterplot_help_button':
            show_help_window(help_text['scatterplot'], location=window.CurrentLocation())
        if event == 'experiment_results_heatmap_help_button':
            show_help_window(help_text['heatmap'], location=window.CurrentLocation())
        if event == 'experiment_results_save_help_button':
            show_help_window(help_text['save_results'], location=window.CurrentLocation())

    def export_experiment_results(self, values, file_path):
        results_dict = {}
        results_dict['Configuration'] = self.results[2]
        results_dict['Parameter names'] = list(self.results[0].columns)
        results_dict['Parameter values'] = self.results[0].values.tolist()
        results_dict['Reporter names'] = list(self.results[1].keys())
        results_dict['Reporter values'] = self.results[1]

        f = open(file_path, "w")
        f.write(str(results_dict))
        f.close()



import pyNetLogo as pnl
import pandas as pd
import math
from pyNetLogo.core import NetLogoException


def run_experiments(child_conn, values, problem, param_values, netlogo_version, netlogo_home):
    outcomes = {}
    repetitions = int(values['experiment_repetition_input'])
    ticks = int(values['experiment_tick_input'])
    steps = 100 / (len(param_values) * repetitions)
    progress_bar = 1

    reporters = split_user_input(values['experiment_reporter_input'])
    for reporter in reporters:
        outcomes[reporter] = []


    try:
        netlogo = pnl.NetLogoLink(gui=False, netlogo_version=netlogo_version, netlogo_home=netlogo_home)
    except (TypeError, KeyError):
        child_conn.send(get_failure_message('Error in input: Invalid NetLogo version or path. Please make sure the '
                                            'values in gui_user.py are correct (this requires a restart of the GUI).'))
        return

    model = values['experiment_model_input']
    try:
        netlogo.load_model(model)
    except (FileNotFoundError, NetLogoException):
        child_conn.send(get_failure_message('Error in input: Invalid model file path. Please make sure the file path is'
                                            ' correct and points to a valid .nlogo file.'))
        return

    for scenario in range(len(param_values)):
        for i, name in enumerate(problem['names']):
            if name == 'random-seed':
                netlogo.command(f'random-seed {param_values[scenario, i]}')
            else:
                try:
                    netlogo.command(f'set {name} {param_values[scenario, i]}')
                except NetLogoException:
                    child_conn.send(get_failure_message('Error in input: Invalid variable bounds input. Please '
                                                        'make sure all variable names and their bounds are correct.'))
                    netlogo.kill_workspace()
                    return

        run_list = {}
        for reporter in reporters:
            run_list[reporter] = []

        for run in range(repetitions):
            setup_commands = split_user_input(values['experiment_setup_input'])
            for command in setup_commands:
                try:
                    netlogo.command(command)
                except NetLogoException:
                    child_conn.send(get_failure_message('Error in input: Invalid setup commands. Please '
                                                        'make sure they\'re all valid NetLogo commands'))
                    netlogo.kill_workspace()
                    return

            try:
                results = netlogo.repeat_report(reporters, ticks)
            except NetLogoException:
                child_conn.send(get_failure_message('Error in input: Invalid reporter commands. Please '
                                                    'make sure all reporters are valid NetLogo commands'))
                netlogo.kill_workspace()
                return

            for reporter in outcomes.keys():
                run_list[reporter].append(list(results[reporter]))

            if (repetitions * scenario + run) * steps > progress_bar:
                progress_bar = math.ceil(progress_bar + steps)
                if steps < 1:
                    child_conn.send({'Progress': 1, 'Results': []})
                else:
                    child_conn.send({'Progress': steps, 'Results': []})
        for reporter in reporters:
            outcomes[reporter].append(run_list[reporter])

    netlogo.kill_workspace()

    results = (param_values, outcomes)
    child_conn.send({'Progress': steps, 'Results': results})


def split_user_input(input):
    rows = input.split('\n')
    rows = list(filter(('').__ne__, rows))
    mapped_rows = list(map(lambda x: x.strip(), rows))
    return mapped_rows

def get_failure_message(msg):
    return {'Progress': 'Failure', 'Results': msg}



import PySimpleGUI as sg
from NetLogoDOE.src.util.connectors.experimental_connector import run_experiments
from NetLogoDOE.src.util.connectors.standard_connector import run_standard
from NetLogoDOE.src.util.Runner import Runner
from NetLogoDOE.src.util.config_dicts.get_experiment_dict import get_experiment_config_dictionary
from NetLogoDOE.src.util.config_dicts.get_standard_dict import get_standard_config_dictionary
import pandas as pd


class RunScreen:

    def __init__(self, netlogo_version, netlogo_home):
        self.layout = [[sg.Text("Running experiments...")],
                       [sg.ProgressBar(100, size=(20, 20), orientation='h', key='run_progressbar')]]
        self.netlogo_version = netlogo_version
        self.netlogo_home = netlogo_home

    def check_events(self, event, values, window):
        if event == 'experiment_run_signal':
            runner_list = []
            process_amount = int(values['experiment_process_input'])

            problem = values['experiment_run_signal'][0]
            experiments = values['experiment_run_signal'][1]
            exp_per_process = [len(experiments) // process_amount + (1 if x < len(experiments) % process_amount else 0)
                                for x in range(process_amount)]
            frames = [experiments[i:i + exp_per_process[i]] for i in range(0, len(exp_per_process))]

            for i in range(process_amount):
                runner = Runner(run_experiments, (values, problem, frames[i], self.netlogo_version, self.netlogo_home))
                runner_list.append(runner)
                runner.run()

            results = []
            progress = 0
            while runner_list:
                for i, runner in enumerate(runner_list):
                    res = runner.get_result()
                    if res is None:
                        window.read(timeout=0.01)
                        continue
                    elif res['Progress'] == 'Failure':
                        window['experiment_panel'].update(visible=True)
                        window['run_panel'].update(visible=False)
                        window.write_event_value('show_error_window', res['Results'])
                        return
                    elif res['Results']:
                        results.append(res)
                        runner_list.remove(runner)
                    progress += (res["Progress"] / process_amount)
                    window['run_progressbar'].update_bar(progress)

            data = self.merge_experiment_data(results, problem['names'])
            config = self.get_experiment_config(values)
            window.write_event_value('experiment_write_results_event', data + (config,))
            window['experiment_result_panel'].update(visible=True)
            window['run_panel'].update(visible=False)
            window['run_progressbar'].update_bar(0)

        if event == 'standard_run_signal':
            runner_list = []
            process_amount = int(values['standard_process_input'])
            repetitions = int(values['standard_repetition_input'])
            reps_per_process = [repetitions // process_amount + (1 if x < repetitions % process_amount else 0)
                                for x in range(process_amount)]

            for i in range(process_amount):
                runner = Runner(run_standard, (values, reps_per_process[i], self.netlogo_version, self.netlogo_home))
                runner_list.append(runner)
                runner.run()

            results = []
            progress = 0
            while runner_list:
                for i, runner in enumerate(runner_list):
                    res = runner.get_result()
                    if res is None:
                        window.read(timeout=0.01)
                        continue
                    elif res['Progress'] == 'Failure':
                        window['standard_panel'].update(visible=True)
                        window['run_panel'].update(visible=False)
                        window.write_event_value('show_error_window', res['Results'])
                        return
                    elif res['Results']:
                        results.append(res)
                        runner_list.remove(runner)
                    progress += (res["Progress"] / process_amount)
                    window['run_progressbar'].update_bar(progress)

            data = self.merge_standard_data(results)
            config = self.get_standard_config(values)
            window.write_event_value('standard_write_results_event', data + (config,))
            window['standard_result_panel'].update(visible=True)
            window['run_panel'].update(visible=False)
            window['run_progressbar'].update_bar(0)

    def merge_experiment_data(self, results, names):
        results = list(map(lambda x: {'Results': x['Results']}, results))
        experiments = []
        outcomes = {}

        for key in results[0]['Results'][1].keys():
            outcomes[key] = []

        for process in results:
            for exp in process['Results'][0]:
                experiments.append(exp)
            for key in process['Results'][1].keys():
                for scenario in process['Results'][1][key]:
                    outcomes[key].append(scenario)

        experiments_df = pd.DataFrame(experiments, columns=names)
        return experiments_df, outcomes

    def merge_standard_data(self, results):
        results = list(map(lambda x: {'Results': x['Results']}, results))
        var_values = results[0]['Results'][0]
        outcomes = {}

        for key in results[0]['Results'][1].keys():
            outcomes[key] = []

        for process in results:
            for key in outcomes.keys():
                for repetition in process['Results'][1][key]:
                    outcomes[key].append(repetition)

        return var_values, outcomes

    def get_experiment_config(self, values):
        experiment_dict = get_experiment_config_dictionary(values)

        table_list = []
        for key in experiment_dict.keys():
            table_list.append([key, experiment_dict[key]])
        return table_list

    def get_standard_config(self, values):
        standard_dict = get_standard_config_dictionary(values)

        table_list = []
        for key in standard_dict.keys():
            table_list.append([key, standard_dict[key]])
        return table_list

    def handle_error(self, window, msg):
        window['experiment_panel'].update(visible=True)
        window['run_panel'].update(visible=False)
        window.write_event_value('show_error_window', msg)



import PySimpleGUI as sg
import plotly.express as px
from NetLogoDOE.src.util.data_processing.merge_experiment_data import merge_data
from NetLogoDOE.src.gui.custom_components import title, metric4_radio_buttons, question_mark_button
from NetLogoDOE.src.gui.custom_windows import show_help_window
from NetLogoDOE.src.gui.help_dictionary import help_text


class ParallelCoordinatesScreen:

    def __init__(self):
        self.layout = [[title('Parallel Coordinates')],
                       [sg.Text('Graph Title:'), sg.Input(key='parcoords_title_input')],
                       [sg.Text('Line Color:'), sg.Input('Blue', key='parcoords_line_color_input')],
                       [sg.Text('Background Color:'), sg.Input('White', key='parcoords_background_color_input')],
                       [question_mark_button('parcoords_4metric_help_button'), sg.Text('Metric:')],
                       metric4_radio_buttons('parcoords'),
                       [sg.Button('Generate', key='parcoords_generate_button')],
                       [sg.Button('Back', key='parcoords_back_button')]]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'experiment_write_results_event':
            self.results = values['experiment_write_results_event']
        if event == 'parcoords_generate_button':
            self.generate_parallel_coordinates(values, window)
        if event == 'parcoords_back_button':
            window['parcoords_panel'].update(visible=False)
            window['experiment_result_panel'].update(visible=True)

        # Help events
        if event == 'parcoords_4metric_help_button':
            show_help_window(help_text['4_metric'], location=window.CurrentLocation())

    def generate_parallel_coordinates(self, values, window):
        exp = self.results[0]
        outcomes = self.results[1]

        repetitions = len(self.results[1][list(self.results[1].keys())[0]][0])
        metric = self.get_selected_radio_button(values)
        merged_data = merge_data(exp, outcomes, repetitions, metric)

        fig = px.parallel_coordinates(merged_data, title=values['parcoords_title_input'])
        try:
            fig = fig.update_layout(
                colorway=[values['parcoords_line_color_input'].lower()],
                plot_bgcolor=values['parcoords_background_color_input'],
                paper_bgcolor=values['parcoords_background_color_input'],
                title_x=0.5,
            )
            fig.show()
        except ValueError:
            window.write_event_value('show_error_window', 'Error in input: Invalid color value for the line or '
                                                          'background.\nPlease make sure the input is a valid CSS '
                                                          'color string (black), hexidecimal (#FFFFFF) or rgb value '
                                                          '(rbg(255, 255, 255))')

    def get_selected_radio_button(self, values):
        for i in range(4):
            if values[f'parcoords_metric_{i + 1}']:
                return i + 1
        return 1



import PySimpleGUI as sg
import plotly.graph_objects as go
from NetLogoDOE.src.util.data_processing.merge_standard_data import merge_data
import plotly.figure_factory as ff
from NetLogoDOE.src.gui.custom_components import title, metric3_radio_buttons, question_mark_button
from NetLogoDOE.src.gui.custom_windows import show_help_window
from NetLogoDOE.src.gui.help_dictionary import help_text


class DistributionplotScreen:

    def __init__(self):
        self.layout = [[title('Distribution plot')],
                       [sg.Text('Graph Title:'), sg.Input('', key='distplot_title_input')],
                       [question_mark_button('distplot_reporter_help_button'), sg.Text('Reporters to plot:')],
                       [sg.Multiline('', key='distplot_reporter_input')],
                       [sg.Text('Only input a single reporter on each line')],
                       [question_mark_button('distplot_3metric_help_button'), sg.Text('Metric:')],
                       metric3_radio_buttons('distplot'),
                       [question_mark_button('distplot_indicators_help_button'), sg.Text('Distribution indicators:')],
                       [sg.Checkbox('Show curve', key='distplot_curve_checkbox', default=True),
                       sg.Checkbox('Show histogram', key='distplot_histogram_checkbox', default=True),
                       sg.Checkbox('Show rug', key='distplot_rug_checkbox', default=True)],
                       [sg.Button('Generate', key='distplot_generate_button')],
                       [sg.Button('Back', key='distplot_back_button')]]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'standard_write_results_event':
            self.results = values['standard_write_results_event']
        if event == 'distplot_generate_button':
            valid, error_message = self.validate_user_input(values)
            if not valid:
                window.write_event_value('show_error_window', error_message)
                return
            self.generate_histogram(values, window)
        if event == 'distplot_back_button':
            window['distplot_panel'].update(visible=False)
            window['standard_result_panel'].update(visible=True)

        # Help events
        if event == 'distplot_reporter_help_button':
            show_help_window(help_text['standard_plot_reporters'], location=window.CurrentLocation())
        if event == 'distplot_3metric_help_button':
            show_help_window(help_text['3_metric'], location=window.CurrentLocation())
        if event == 'distplot_indicators_help_button':
            show_help_window(help_text['distributionplot_indicators'], location=window.CurrentLocation())

    def generate_histogram(self, values, window):
        reporters = self.format_reporters(values)
        metric = self.get_selected_radio_button(values)
        merged_data = merge_data(self.results[1], metric)
        try:
            [merged_data[reporter] for reporter in reporters]
        except KeyError:
            window.write_event_value('show_error_window', 'Error in input: Invalid reporter name.\nPlease make '
                                                          'sure all reporters are valid with the current run '
                                                          'configuration.')
            return

        hist_data = []
        for reporter in reporters:
            hist_data.append(merged_data[reporter])

        bin_size = [(max(data)-min(data))/30 for data in hist_data]
        fig = ff.create_distplot(hist_data, reporters, bin_size,
                                 show_curve=values['distplot_curve_checkbox'],
                                 show_hist=values['distplot_histogram_checkbox'],
                                 show_rug=values['distplot_rug_checkbox'],
                                 )
        fig.update_layout(title_text=values['distplot_title_input'],
                          xaxis_title_text='Value',
                          )
        fig.show()

    def get_selected_radio_button(self, values):
        for i in range(3):
            if values[f'distplot_metric_{i + 1}']:
                return i + 1
        return 1

    def format_reporters(self, values):
        reporters = list(filter(('').__ne__, values['distplot_reporter_input'].split('\n')))
        reporters = list(map(lambda x: x.strip(), reporters))
        return reporters

    def validate_user_input(self, values):
        if values['distplot_reporter_input'] == '\n':
            return False, 'Error in input: One or more fields are empty. ' \
                          'Please make sure all options are filled out.'

        return True, ''



import PySimpleGUI as sg
import ast
import pandas as pd
from NetLogoDOE.src.gui.custom_components import title, explanation, question_mark_button
from NetLogoDOE.src.gui.custom_windows import show_help_window
from NetLogoDOE.src.gui.help_dictionary import help_text


class ImportScreen:

    def __init__(self):
        button_size = (25, 1)
        button_pad = ((5, 0), (20, 0))
        self.layout = [[title('Import results')],
                       [sg.Input(key='import_dummy_experiment_results', enable_events=True, visible=False, size=(0, 0)),
                        sg.FileBrowse('Import experiment results', file_types=[("Text Files", "*.txt")],
                                      target='import_dummy_experiment_results',
                                      key='import_experiment_results_button', size=button_size, pad=button_pad),
                        question_mark_button('import_experiment_results_help_button', padding=button_pad)],
                       [explanation('whatever needs ot be said')],
                       [sg.Input(key='import_dummy_standard_results', enable_events=True, visible=False, size=(0, 0)),
                        sg.FileBrowse('Import standard results', file_types=[("Text Files", "*.txt")],
                                      target='import_dummy_standard_results',
                                      key='import_standard_results_button', size=button_size, pad=button_pad),
                        question_mark_button('import_standard_results_help_button', padding=button_pad)],
                       [explanation('whatever needs ot be said')],
                       [sg.Input(key='import_dummy_behaviorspace_results', enable_events=True, visible=False,
                                 size=(0, 0)),
                        sg.FileBrowse('Import BehaviorSpace results', file_types=[("Text Files", "*.txt")],
                                      target='import_dummy_behaviorspace_results',
                                      key='import_behaviorspace_results_button', size=button_size, pad=button_pad),
                        question_mark_button('import_behaviorspace_results_help_button', padding=button_pad)],
                       [explanation('whatever needs ot be said')],
                       [sg.Button('Back', key='import_back_button', pad=button_pad)]
                       ]

    def check_events(self, event, values, window):
        if event == 'import_dummy_experiment_results' and not (values['import_dummy_experiment_results'] == ''):
            self.import_experiment_results(values['import_dummy_experiment_results'], window)
        if event == 'import_dummy_standard_results' and not (values['import_dummy_standard_results'] == ''):
            self.import_standard_results(values['import_dummy_standard_results'], window)
        if event == 'import_dummy_behaviorspace_results' and not (values['import_dummy_behaviorspace_results'] == ''):
            self.import_behaviorspace_results(values['import_dummy_behaviorspace_results'], window)
        if event == 'import_back_button':
            window['main_panel'].update(visible=True)
            window['import_panel'].update(visible=False)

        # Help events
        if event == 'import_experiment_results_help_button':
            show_help_window(help_text['import_experiment'], location=window.CurrentLocation())
        if event == 'import_standard_results_help_button':
            show_help_window(help_text['import_standard'], location=window.CurrentLocation())
        if event == 'import_behaviorspace_results_help_button':
            show_help_window(help_text['import_behaviorspace'], location=window.CurrentLocation())


    def import_experiment_results(self, file_path, window):
        f = open(file_path, "r")
        experiment_string = f.read()
        f.close()

        try:
            experiment_dict = ast.literal_eval(experiment_string)
            config = experiment_dict['Configuration']
            exp = pd.DataFrame(experiment_dict['Parameter values'], columns=experiment_dict['Parameter names'])
            outcomes = experiment_dict['Reporter values']
        except (SyntaxError, KeyError):
            window.write_event_value('show_error_window', 'Invalid file syntax for experimental results')
            return

        window.write_event_value('experiment_write_results_event', (exp, outcomes, config))
        window['import_panel'].update(visible=False)
        window['experiment_result_panel'].update(visible=True)

    def import_standard_results(self, file_path, window):
        f = open(file_path, "r")
        standard_string = f.read()
        f.close()

        try:
            standard_dict = ast.literal_eval(standard_string)
            config = standard_dict['Configuration']
            results = (standard_dict['Parameter settings'], standard_dict['Reporter values'], config)
        except (SyntaxError, KeyError):
            window.write_event_value('show_error_window', 'Invalid file syntax for standard results')
            return

        window.write_event_value('standard_write_results_event', results)
        window['import_panel'].update(visible=False)
        window['standard_result_panel'].update(visible=True)

    def import_behaviorspace_results(self, file_path, window):
        print(file_path)
        print('Not implemented yet')



import PySimpleGUI as sg
import ast
from NetLogoDOE.src.util.Sampler import MonteCarloSampler, LatinHypercubeSampler, FullFactorialSampler, FASTSampler, \
    FiniteDifferenceSampler, SaltelliSampler, SobolSampler
from NetLogoDOE.src.util.config_dicts.get_experiment_dict import get_experiment_config_dictionary
from NetLogoDOE.src.gui.custom_components import title, question_mark_button
from NetLogoDOE.src.gui.help_dictionary import help_text
from NetLogoDOE.src.gui.custom_windows import show_help_window



class ExperimentScreen:

    def __init__(self):
        question_mark_padding = ((0, 0), (0, 0))
        self.layout = [[title('Experimental runs')],
                       [sg.Text('Experiment Name'), sg.Input(key='experiment_name_input'),
                        sg.Input(key='experiment_dummy_import', enable_events=True, visible=False, size=(0, 0)),
                        sg.FileBrowse('Import experiment', file_types=[("Text Files", "*.txt")],
                                      target='experiment_dummy_import', key='experiment_import_button')],
                       [sg.Text('Model file name'), sg.Input(key='experiment_model_input'),
                        sg.Input(key='experiment_model_dummy_import', enable_events=True, visible=False, size=(0, 0)),
                        sg.FileBrowse('Import model', file_types=[("NetLogo Files", "*.nlogo")],
                                      target='experiment_model_dummy_import', key='experiment_model_import_button')],
                       [question_mark_button('experiment_bound_input_help_button'), sg.Text('Vary variables as follows:')],
                       [sg.Multiline(key='experiment_bound_input')],
                       [sg.Text('Explain the format: variable-name lower-bound upper-bound')],
                       [question_mark_button('experiment_sampling_help_button'), sg.Text('Sampling method:')],
                       [sg.Radio('Monte Carlo', 'experiment_sampler_radiogroup', key='sample_mc'),
                        sg.Radio('Latin Hypercube', 'experiment_sampler_radiogroup', key='sample_lhs'),
                        sg.Radio('Full Factorial', 'experiment_sampler_radiogroup', key='sample_ff'),
                        sg.Radio('FAST', 'experiment_sampler_radiogroup', key='sample_fast'),
                        # sg.Radio('Finite Difference', 'experiment_sampler_radiogroup', key='sample_fd'),
                        #                         question_mark('Help', padding=question_mark_padding),
                        sg.Radio('Saltelli', 'experiment_sampler_radiogroup', key='sample_saltelli'),
                        sg.Radio('Sobol sequence', 'experiment_sampler_radiogroup', key='sample_ss')],
                       [question_mark_button('experiment_scenario_help_button'), sg.Text('Number of scenarios:'),
                        sg.Input('10', key='experiment_scenario_input')],
                       [question_mark_button('experiment_repetition_help_button'), sg.Text('Number of repetitions per scenario:'),
                        sg.Input('10', key='experiment_repetition_input')],
                       [question_mark_button('experiment_tick_help_button'), sg.Text('Maximum number of ticks per run:'),
                        sg.Input('100', key='experiment_tick_input')],
                       [question_mark_button('experiment_reporter_help_button'), sg.Text('Measure runs using these reporters:')],
                       [sg.Multiline(key='experiment_reporter_input')],
                       [sg.Text('Only input a single reporter on each line')],
                       [question_mark_button('experiment_setup_help_button'), sg.Text('Setup commands:')],
                       [sg.Multiline('setup', key='experiment_setup_input')],
                       [question_mark_button('experiment_process_help_button'), sg.Text('Number of parallel executors:'),
                        sg.Input('2', key='experiment_process_input')],
                       [sg.Button('Run', key="experiment_run_button")],
                       [sg.Button('Back', key="experiment_back_button"),
                        sg.Input(key='experiment_dummy_export', enable_events=True, visible=False, size=(0, 0)),
                        sg.SaveAs('Save Experiment', file_types=[("Text Files", "*.txt")],
                                  target='experiment_dummy_export', key="experiment_save_button")]]

    def check_events(self, event, values, window):
        if event == 'experiment_run_button':
            valid, error_message = self.validate_user_input(values)
            if not valid:
                window.write_event_value('show_error_window', error_message)
                return

            window['experiment_panel'].update(visible=False)
            window['run_panel'].update(visible=True)
            window.read(0.01)

            problem = self.construct_problem(values)
            param_values = self.get_param_values(problem, values)
            window.write_event_value('experiment_run_signal', (problem, param_values))

        if event == 'experiment_dummy_import' and not (values['experiment_dummy_import'] == ''):
            self.import_experiment(window, values['experiment_dummy_import'])
        if event == 'experiment_model_dummy_import' and not (values['experiment_model_dummy_import'] == ''):
            window['experiment_model_input'].update(values['experiment_model_dummy_import'])
        if event == 'experiment_dummy_export' and not (values['experiment_dummy_export'] == ''):
            self.export_experiment(values, values['experiment_dummy_export'])
        if event == 'experiment_back_button':
            window['main_panel'].update(visible=True)
            window['experiment_panel'].update(visible=False)

        # Help events
        if event == 'experiment_bound_input_help_button':
            show_help_window(help_text['experiment_variables'], location=window.CurrentLocation())
        if event == 'experiment_sampling_help_button':
            show_help_window(help_text['experiment_sampling'], location=window.CurrentLocation())
        if event == 'experiment_scenario_help_button':
            show_help_window(help_text['experiment_scenarios'], location=window.CurrentLocation())
        if event == 'experiment_repetition_help_button':
            show_help_window(help_text['run_repetitions'], location=window.CurrentLocation())
        if event == 'experiment_tick_help_button':
            show_help_window(help_text['run_ticks'], location=window.CurrentLocation())
        if event == 'experiment_reporter_help_button':
            show_help_window(help_text['run_reporters'], location=window.CurrentLocation())
        if event == 'experiment_setup_help_button':
            show_help_window(help_text['run_setup'], location=window.CurrentLocation())
        if event == 'experiment_process_help_button':
            show_help_window(help_text['run_processes'], location=window.CurrentLocation())

    def construct_problem(self, values):
        num_vars, names, bounds = self.get_problem_values(values['experiment_bound_input'])
        return {
            'num_vars': num_vars,
            'names': names,
            'bounds': bounds
        }

    def get_problem_values(self, bounds):
        rows = bounds.split('\n')
        rows = list(filter(('').__ne__, rows))
        split_rows = list(map(lambda x: x.split(' '), rows))
        names = list(map(lambda x: x[0], split_rows))
        bounds = list(map(lambda x: [int(x[1]), int(x[2])], split_rows))
        return len(rows), names, bounds

    def get_param_values(self, problem, values):
        keys = ['sample_mc', 'sample_lhs', 'sample_ff', 'sample_fast', 'sample_saltelli', 'sample_ss']#, 'sample_fd']
        scenarios = int(values['experiment_scenario_input'])
        for key in keys:
            if values[key]:
                param_values = []
                if key == 'sample_mc':
                    param_values = MonteCarloSampler().sample(problem, scenarios)
                elif key == 'sample_lhs':
                    param_values = LatinHypercubeSampler().sample(problem, scenarios)
                elif key == 'sample_ff':
                    param_values = FullFactorialSampler().sample(problem, scenarios)
                elif key == 'sample_fast':
                    param_values = FASTSampler().sample(problem, scenarios)
                elif key == 'sample_fd':
                    param_values = FiniteDifferenceSampler().sample(problem, scenarios)
                elif key == 'sample_saltelli':
                    param_values = SaltelliSampler().sample(problem, scenarios)
                elif key == 'sample_ss':
                    param_values = SobolSampler().sample(problem, scenarios)
                return param_values
        return LatinHypercubeSampler().sample(problem, scenarios)

    def import_experiment(self, window, file_path):
        f = open(file_path, "r")
        experiment_string = f.read()
        f.close()

        try:
            experiment_dict = ast.literal_eval(experiment_string)
            window['experiment_name_input'].update(experiment_dict['Experiment Name'])
            window['experiment_model_input'].update(experiment_dict['Model file'])
            window['experiment_bound_input'].update('\n'.join((experiment_dict['Variable bounds'])))
            window['experiment_scenario_input'].update(experiment_dict['Number of scenarios'])
            window['experiment_repetition_input'].update(experiment_dict['Repetitions'])
            window['experiment_tick_input'].update(experiment_dict['Ticks per run'])
            window['experiment_reporter_input'].update('\n'.join(experiment_dict['NetLogo reporters']))
            window['experiment_setup_input'].update('\n'.join(experiment_dict['Setup commands']))
            window['experiment_process_input'].update(experiment_dict['Parallel executors'])
        except (SyntaxError, KeyError, TypeError, IndexError):
            window.write_event_value('show_error_window', 'Invalid file syntax for experiment configuration')
            return

    def export_experiment(self, values, file_path):
        experiment_dict = get_experiment_config_dictionary(values)

        f = open(file_path, "w")
        f.write(str(experiment_dict))
        f.close()

    def validate_user_input(self, values):
        if values['experiment_model_input'] == '' or \
                values['experiment_bound_input'] == '\n' or \
                values['experiment_scenario_input'] == '' or \
                values['experiment_repetition_input'] == '' or \
                values['experiment_tick_input'] == '' or \
                values['experiment_reporter_input'] == '\n' or \
                values['experiment_setup_input'] == '\n' or \
                values['experiment_process_input'] == '':
            return False, 'Error in input: One or more field are empty. ' \
                          'Please make sure all options are filled out.'

        try:
            problem = self.construct_problem(values)
        except (IndexError, ValueError):
            return False, 'Error in input: Invalid variable bounds input. ' \
                          'Please make sure the correct format is used.'
        try:
            scenarios = int(values['experiment_scenario_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the number of scenarios. ' \
                          'Please make sure it\'s an integer.'
        try:
            repetitions = int(values['experiment_repetition_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the number of repetitions. ' \
                          'Please make sure it\'s an integer.'
        try:
            repetitions = int(values['experiment_tick_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the maximum number of ticks. ' \
                          'Please make sure it\'s an integer.'
        try:
            process_amount = int(values['experiment_process_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the number of executors. Please make sure it\'s an integer.'

        return True, ''



def get_standard_config_dictionary(values):
    run_dict = {}
    run_dict['Configuration Name'] = values['standard_name_input']
    run_dict['Model file'] = values['standard_model_input']
    var_values = list(filter(''.__ne__, values['standard_value_input'].split('\n')))
    run_dict['Variable values'] = list(map(lambda x: x.strip(), var_values))
    run_dict['Repetitions'] = values['standard_repetition_input']
    run_dict['Ticks per run'] = values['standard_tick_input']
    reporters = list(filter(''.__ne__, values['standard_reporter_input'].split('\n')))
    run_dict['NetLogo reporters'] = list(map(lambda x: x.strip(), reporters))
    setup_commands = list(filter(''.__ne__, values['standard_setup_input'].split('\n')))
    run_dict['Setup commands'] = list(map(lambda x: x.strip(), setup_commands))
    run_dict['Parallel executors'] = values['standard_process_input']
    return run_dict


help_text = {
    # Index screen
    'experimental_runs':'Design an experiment that runs different parameter configurations by specifying parameter '
                        'bounds.\nThe results can be visualised with parallel coordinates, scatter plots and heatmaps.',
    'standard_runs':'Help1',
    'import_results':'Help2',

    # Import screen
    'import_experiment':'Help3',
    'import_standard':'Help4',
    'import_behaviorspace':'Help5',

    # Experiment/Standard screen
    'run_repetitions': 'Help6',
    'run_ticks': 'Help7',
    'run_reporters': 'Help8',
    'run_setup': 'Help9',
    'run_processes': 'Help10',
    'save_config':'Help11',

    # Experiment screen
    'experiment_variables':'Help12',
    'experiment_sampling':'Help13',
    'experiment_scenarios':'Help14',

    # Standard screen
    'standard_variables':'Help15',

    # Experiment/Standard plots
    'config_information':'Help',
    'save_results':'Help',

    # Experiment plots
    'parallel_coordinates':'Help',
    'scatterplot':'Help',
    'scatterplot_variables':'Help',
    'heatmap':'Help',
    '4_metric':'Help',

    # Standard plots
    'timeseries':'Help',
    'boxplot':'Help',
    'violinplot':'Help',
    'histogram':'Help',
    'distributionplot':'Help',
    'distributionplot_indicators':'Help',
    '3_metric':'Help',
    'standard_plot_reporters':'Help',

}



def get_experiment_config_dictionary(values):
    experiment_dict = {}
    experiment_dict['Experiment Name'] = values['experiment_name_input']
    experiment_dict['Model file'] = values['experiment_model_input']
    bounds = list(filter(''.__ne__, values['experiment_bound_input'].split('\n')))
    experiment_dict['Variable bounds'] = list(map(lambda x: x.strip(), bounds))
    experiment_dict['Number of scenarios'] = values['experiment_scenario_input']
    experiment_dict['Repetitions'] = values['experiment_repetition_input']
    experiment_dict['Ticks per run'] = values['experiment_tick_input']
    reporters = list(filter(''.__ne__, values['experiment_reporter_input'].split('\n')))
    experiment_dict['NetLogo reporters'] = list(map(lambda x: x.strip(), reporters))
    setup_commands = list(filter(''.__ne__, values['experiment_setup_input'].split('\n')))
    experiment_dict['Setup commands'] = list(map(lambda x: x.strip(), setup_commands))
    experiment_dict['Parallel executors'] = values['experiment_process_input']
    return experiment_dict


import PySimpleGUI as sg
import ast
from NetLogoDOE.src.util.config_dicts.get_standard_dict import get_standard_config_dictionary
from NetLogoDOE.src.gui.custom_components import title, question_mark_button
from NetLogoDOE.src.gui.help_dictionary import help_text
from NetLogoDOE.src.gui.custom_windows import show_help_window

class StandardScreen:

    def __init__(self):
        question_mark_padding = ((0, 0), (0, 0))
        self.layout = [[title('Standard runs')],
                       [sg.Text('Configuration name'), sg.Input(key='standard_name_input'),
                        sg.Input(key='standard_dummy_import', enable_events=True, visible=False, size=(0, 0)),
                        sg.FileBrowse('Import Run Configuration', file_types=[("Text Files", "*.txt")],
                                      target='standard_dummy_import', key='standard_import_button')],
                       [sg.Text('Model file name'), sg.Input(key='standard_model_input'),
                        sg.Input(key='standard_model_dummy_import', enable_events=True, visible=False, size=(0, 0)),
                        sg.FileBrowse('Import model', file_types=[("NetLogo Files", "*.nlogo")],
                                      target='standard_model_dummy_import', key='standard_model_import_button')],
                       [question_mark_button('standard_value_help_button'),
                        sg.Text('Set variables as follows:')],
                       [sg.Multiline(key='standard_value_input')],
                       [sg.Text('Explain the format: variable-name variable-value')],
                       [question_mark_button('standard_repetition_help_button'),
                        sg.Text('Number of repetitions:'),
                        sg.Input('10', key='standard_repetition_input')],
                       [question_mark_button('standard_tick_help_button'),
                        sg.Text('Maximum number of ticks per run:'),
                        sg.Input('100', key='standard_tick_input')],
                       [question_mark_button('standard_reporter_help_button'),
                        sg.Text('Measure runs using these reporters:')],
                       [sg.Multiline(key='standard_reporter_input')],
                       [sg.Text('Only input a single reporter on each line')],
                       [question_mark_button('standard_setup_help_button'),
                        sg.Text('Setup commands:')],
                       [sg.Multiline('setup', key='standard_setup_input')],
                       [question_mark_button('standard_process_help_button'),
                        sg.Text('Number of parallel executors:'),
                        sg.Input('2', key='standard_process_input')],
                       [sg.Button('Run', key="standard_run_button")],
                       [sg.Button('Back', key="standard_back_button"),
                        sg.Input(key='standard_dummy_export', enable_events=True, visible=False, size=(0, 0)),
                        sg.SaveAs('Save Run Configuration', file_types=[("Text Files", "*.txt")],
                                  target='standard_dummy_export', key="standard_save_button")]]

    def check_events(self, event, values, window):
        if event == 'standard_run_button':
            valid, error_message = self.validate_user_input(values)
            if not valid:
                window.write_event_value('show_error_window', error_message)
                return

            window['standard_panel'].update(visible=False)
            window['run_panel'].update(visible=True)
            window.read(timeout=0.01)

            window.write_event_value('standard_run_signal', '')
        if event == 'standard_dummy_import' and not (values['standard_dummy_import'] == ''):
            self.import_run(window, values['standard_dummy_import'])
        if event == 'standard_model_dummy_import' and not (values['standard_model_dummy_import'] == ''):
            window['standard_model_input'].update(values['standard_model_dummy_import'])
        if event == 'standard_dummy_export' and not (values['standard_dummy_export'] == ''):
            self.export_run(values, values['standard_dummy_export'])
        if event == 'standard_back_button':
            window['main_panel'].update(visible=True)
            window['standard_panel'].update(visible=False)

        # Help events
        if event == 'standard_value_help_button':
            show_help_window(help_text['standard_variables'], location=window.CurrentLocation())
        if event == 'standard_repetition_help_button':
            show_help_window(help_text['run_repetitions'], location=window.CurrentLocation())
        if event == 'standard_tick_help_button':
            show_help_window(help_text['run_ticks'], location=window.CurrentLocation())
        if event == 'standard_reporter_help_button':
            show_help_window(help_text['run_reporters'], location=window.CurrentLocation())
        if event == 'standard_setup_help_button':
            show_help_window(help_text['run_setup'], location=window.CurrentLocation())
        if event == 'standard_process_help_button':
            show_help_window(help_text['run_processes'], location=window.CurrentLocation())

    def import_run(self, window, file_path):
        f = open(file_path, "r")
        run_string = f.read()
        f.close()

        try:
            run_dict = ast.literal_eval(run_string)
            window['standard_name_input'].update(run_dict['Configuration Name'])
            window['standard_model_input'].update(run_dict['Model file'])
            window['standard_value_input'].update('\n'.join(run_dict['Variable values']))
            window['standard_repetition_input'].update(run_dict['Repetitions'])
            window['standard_tick_input'].update(run_dict['Ticks per run'])
            window['standard_reporter_input'].update('\n'.join(run_dict['NetLogo reporters']))
            window['standard_setup_input'].update('\n'.join(run_dict['Setup commands']))
            window['standard_process_input'].update(run_dict['Parallel executors'])
        except (SyntaxError, KeyError, TypeError, IndexError):
            window.write_event_value('show_error_window', 'Invalid file syntax for standard configuration')
            return

    def export_run(self, values, file_path):
        run_dict = get_standard_config_dictionary(values)

        f = open(file_path, "w")
        f.write(str(run_dict))
        f.close()

    def validate_user_input(self, values):
        if values['standard_model_input'] == '' or \
                values['standard_value_input'] == '\n' or \
                values['standard_repetition_input'] == '' or \
                values['standard_tick_input'] == '' or \
                values['standard_reporter_input'] == '\n' or \
                values['standard_setup_input'] == '\n' or \
                values['standard_process_input'] == '':
            return False, 'Error in input: One or more field are empty. ' \
                          'Please make sure all options are filled out.'

        try:
            repetitions = int(values['standard_repetition_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the number of repetitions. ' \
                          'Please make sure it\'s an integer'
        try:
            repetitions = int(values['standard_tick_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the maximum number of ticks. ' \
                          'Please make sure it\'s an integer'
        try:
            process_amount = int(values['standard_process_input'])
        except ValueError:
            return False, 'Error in input: Invalid value for the number of executors. Please make sure it\'s an integer'

        return True, ''



from abc import abstractmethod
import numpy as np
import math

from pyDOE2 import fullfact
from SALib.sample import latin, fast_sampler, finite_diff, saltelli, sobol_sequence


class AbstractSampler(object):

    def __init__(self):
        super(AbstractSampler, self).__init__()

    @abstractmethod
    def sample(self, problem, samples):
        pass


class MonteCarloSampler(AbstractSampler):

    def __init__(self):
        super(MonteCarloSampler, self).__init__()

    def sample(self, problem, samples):
        parameter_amount = problem['num_vars']
        mcs = np.random.rand(samples, parameter_amount)
        self.normalize(problem, mcs)
        return mcs

    def normalize(self, problem, sampled_data):
        for col in range(len(sampled_data[0])):
            bounds = problem['bounds'][col]
            sampled_data[:, col] = list(map(lambda x: (x * (int(bounds[1]) - int(bounds[0])) + int(bounds[0])),
                                            sampled_data[:, col]))
        return sampled_data


class LatinHypercubeSampler(AbstractSampler):

    def __init__(self):
        super(LatinHypercubeSampler, self).__init__()

    def sample(self, problem, samples):
        return latin.sample(problem, samples)


class FullFactorialSampler(AbstractSampler):

    def __init__(self):
        super(FullFactorialSampler, self).__init__()

    def sample(self, problem, samples):
        ff_numbers = [int(bound[1] - bound[0] + 1) for bound in problem['bounds']]
        ff = fullfact(ff_numbers)
        ff = self.normalize(problem, ff)
        return ff

    def normalize(self, problem, sampled_data):
        for col in range(len(sampled_data[0])):
            bounds = problem['bounds'][col]
            sampled_data[:, col] = list(map(lambda x: (x + int(bounds[0])), sampled_data[:, col]))
        return sampled_data


class FASTSampler(AbstractSampler):

    def __init__(self):
        super(FASTSampler, self).__init__()

    def sample(self, problem, samples):
        N = math.ceil(samples / problem['num_vars'])
        M = math.floor(math.sqrt(N / 4))
        return fast_sampler.sample(problem, max(N, 5), M=max(M, 1))


class FiniteDifferenceSampler(AbstractSampler):

    def __init__(self):
        super(FiniteDifferenceSampler, self).__init__()

    def sample(self, problem, samples):
        return finite_diff.sample(problem, math.ceil(samples/4))


class SaltelliSampler(AbstractSampler):

    def __init__(self):
        super(SaltelliSampler, self).__init__()

    def sample(self, problem, samples):
        n = samples / (2 * problem['num_vars'] + 2)
        return saltelli.sample(problem, math.ceil(n), calc_second_order=True)


class SobolSampler(AbstractSampler):

    def __init__(self):
        super(SobolSampler, self).__init__()

    def sample(self, problem, samples):
        ss = sobol_sequence.sample(samples, problem['num_vars'])
        return self.normalize(problem, ss)

    def normalize(self, problem, sampled_data):
        for col in range(len(sampled_data[0])):
            bounds = problem['bounds'][col]
            sampled_data[:, col] = list(map(lambda x: (x * (int(bounds[1]) - int(bounds[0])) + int(bounds[0])),
                                            sampled_data[:, col]))
        return sampled_data



import PySimpleGUI as sg
import plotly.graph_objects as go
import plotly.express as px
from plotly.colors import hex_to_rgb
import numpy as np
from NetLogoDOE.src.util.data_processing.merge_standard_data import merge_data
from NetLogoDOE.src.gui.custom_components import title, question_mark_button, metric3_radio_buttons
from NetLogoDOE.src.gui.custom_windows import show_help_window
from NetLogoDOE.src.gui.help_dictionary import help_text


class TimeSeriesplotScreen:

    def __init__(self):
        self.layout = [[title('Timeseries plot')],
                       [sg.Text('Graph Title'), sg.Input(key='timeseries_title_input')],
                       [question_mark_button('timeseries_reporter_help_button'), sg.Text('Reporters to plot:')],
                       [sg.Multiline('', key='timeseries_reporter_input')],
                       [sg.Text('Only input a single reporter on each line')],
                       [question_mark_button('timeseries_3metric_help_button'), sg.Text('Metric:')],
                       metric3_radio_buttons('timeseries'),
                       [self.collapse([
                           [sg.Text('Type of variance indication:')],
                           [sg.Radio('None', 'timeseries_variancegroup', key='timeseries_variance_1'),
                            sg.Radio('Error Bars', 'timeseries_variancegroup', key='timeseries_variance_2'),
                            sg.Radio('Shaded areas', 'timeseries_variancegroup', key='timeseries_variance_3')]],
                           'timeseries_variance_radiobuttons')],
                       [sg.Button('Generate', key='timeseries_generate_button')],
                       [sg.Button('Back', key='timeseries_back_button')]]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'standard_write_results_event':
            self.results = values['standard_write_results_event']

        if event == 'timeseries_metric_1':
            window['timeseries_variance_radiobuttons'].update(visible=True)
        if event in ('timeseries_metric_2', 'timeseries_metric_3'):
            window['timeseries_variance_radiobuttons'].update(visible=False)

        if event == 'timeseries_generate_button':
            self.generate_timeseries(values, window)
        if event == 'timeseries_back_button':
            window['timeseries_panel'].update(visible=False)
            window['standard_result_panel'].update(visible=True)

        # Help events
        if event == 'timeseries_reporter_help_button':
            show_help_window(help_text['standard_plot_reporters'], location=window.CurrentLocation())
        if event == 'timeseries_3metric_help_button':
            show_help_window(help_text['3_metric'], location=window.CurrentLocation())

    def collapse(self, layout, key):
        return sg.pin(sg.Column(layout, key=key, visible=False))

    def generate_timeseries(self, values, window):
        reporters = self.format_reporters(values)
        metric = self.get_selected_radio_button(values, 'timeseries_metric', 3)
        variance = self.get_selected_radio_button(values, 'timeseries_variance', 3)
        merged_data = merge_data(self.results[1], metric)

        try:
            [merged_data[reporter] for reporter in reporters]
        except KeyError:
            window.write_event_value('show_error_window', 'Error in input: Invalid reporter name.\nPlease make '
                                                          'sure all reporters are valid with the current run '
                                                          'configuration.')
            return

        if variance > 1 and metric == 1:
            fig = self.generate_variance_figure(merged_data, variance, reporters)
        else:
            fig = go.Figure()
            for reporter in reporters:
                fig.add_trace(go.Scatter(y=merged_data[reporter], mode='lines', name=reporter))

        fig.update_layout(title=values['timeseries_title_input'])
        fig.update_xaxes(title_text='Ticks')
        fig.update_yaxes(title_text='Value')
        fig.show()

    def generate_variance_figure(self, merged_data, variance, reporters):
        fig = go.Figure()
        for reporter in reporters:
            zipped = list(zip(*self.results[1][reporter]))
            zipped_std = list(map(lambda x: np.std(x), zipped))

            if variance == 2:
                fig.add_trace(go.Scatter(y=merged_data[reporter], mode='lines', name=reporter,
                                         error_y=dict(
                                             type='data',
                                             symmetric=False,
                                             array=zipped_std,
                                             arrayminus=zipped_std)
                                         ))
            if variance == 3:
                plus_std = [x + zipped_std[i] for i, x in enumerate(merged_data[reporter])]
                minus_std = [x - zipped_std[i] for i, x in enumerate(merged_data[reporter])]
                fig.add_trace(go.Scatter(y=merged_data[reporter], mode='lines', name=reporter))
                fig.add_trace(go.Scatter(y=plus_std, showlegend=False,
                                         line=dict(width=0), hoverinfo='skip'))
                fig.add_trace(go.Scatter(y=minus_std, showlegend=False,
                                         fill='tonexty',
                                         fillcolor=f"rgba{(*hex_to_rgb(px.colors.qualitative.Plotly[0]), 0.2)}",
                                         line=dict(width=0), hoverinfo='skip'))

        return fig

    def get_selected_radio_button(self, values, prefix, n):
        for i in range(n):
            if values[f'{prefix}_{i + 1}']:
                return i + 1
        return 1

    def format_reporters(self, values):
        reporters = list(filter(('').__ne__, values['timeseries_reporter_input'].split('\n')))
        reporters = list(map(lambda x: x.strip(), reporters))
        return reporters

    def validate_user_input(self, values):
        if values['timeseries_reporter_input'] == '\n':
            return False, 'Error in input: One or more fields are empty. ' \
                          'Please make sure all options are filled out.'

        return True, ''



import PySimpleGUI as sg
from NetLogoDOE.src.gui.custom_components import header, explanation, question_mark_button
from NetLogoDOE.src.gui.help_dictionary import help_text
from NetLogoDOE.src.gui.custom_windows import show_help_window


class MainScreen:

    def __init__(self):
        button_size = (25, 1)
        button_pad = ((5, 0), (20, 0))
        self.layout = [[header('NetLogoDOE')],
                       [sg.Button('Experimental runs', key='main_experiment_button', size=button_size, pad=button_pad),
                        question_mark_button('main_experiment_help_button', padding=button_pad)],
                       [explanation('whatever needs ot be said')],
                       [sg.Button('Standard runs', key='main_standard_button', size=button_size, pad=button_pad),
                        question_mark_button('main_standard_help_button', padding=button_pad)],
                       [explanation('whatever needs ot be said')],
                       [sg.Button('Import results', key='main_import_button', size=button_size, pad=button_pad),
                        question_mark_button('main_import_help_button', padding=button_pad)],
                       [explanation('whatever needs ot be said')],
                       [sg.Button('Close', key='main_close_button', pad=button_pad)]]

    def check_events(self, event, values, window):
        if event == 'main_experiment_button':
            window['main_panel'].update(visible=False)
            window['experiment_panel'].update(visible=True)
        if event == 'main_standard_button':
            window['main_panel'].update(visible=False)
            window['standard_panel'].update(visible=True)
        if event == 'main_import_button':
            window['main_panel'].update(visible=False)
            window['import_panel'].update(visible=True)

        # Help events
        if event == 'main_experiment_help_button':
            show_help_window(help_text['experimental_runs'], location=window.CurrentLocation())
        if event == 'main_standard_help_button':
            show_help_window(help_text['standard_runs'], location=window.CurrentLocation())
        if event == 'main_import_help_button':
            show_help_window(help_text['import_results'], location=window.CurrentLocation())



import numpy as np


def merge_data(data, outcomes, repetitions, metric):
    for key in outcomes.keys():
        scenario_avgs = []
        for scenario in range(len(outcomes[key])):
            results = []
            for replication in range(repetitions):
                if metric == 1:
                    results.append(np.mean(outcomes[key][scenario][replication]))
                if metric == 2:
                    results.append(min(outcomes[key][scenario][replication]))
                if metric == 3:
                    results.append(max(outcomes[key][scenario][replication]))
                if metric == 4:
                    results.append(outcomes[key][scenario][replication][-1])
            scenario_avgs.append(np.mean(results))

        data[key] = scenario_avgs
    return data



import PySimpleGUI as sg
from NetLogoDOE.src.gui.custom_components import title, configuration_parameter_text, configuration_value_text, configuration_horizontal_line


class StandardConfigTableScreen:

    def __init__(self):
        self.layout = [[title('Standard Configuration')],
                       [configuration_parameter_text('Configuration Name:'),
                        configuration_value_text('', 'standard_configtable_configuration_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('Model file:'),
                        configuration_value_text('', 'standard_configtable_model_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('Parameter bounds:'),
                        configuration_value_text('', 'standard_configtable_variable_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('Repetitions:'),
                        configuration_value_text('', 'standard_configtable_repetition_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('Ticks per run:'),
                        configuration_value_text('', 'standard_configtable_tick_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('NetLogo reporters:'),
                        configuration_value_text('', 'standard_configtable_reporter_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('Setup commands:'),
                        configuration_value_text('', 'standard_configtable_setup_text')],
                       [configuration_horizontal_line()],
                       [configuration_parameter_text('Parallel executors:'),
                        configuration_value_text('', 'standard_configtable_process_text')],
                       [configuration_horizontal_line()],
                       [sg.Button('Back', key='standard_configtable_back_button')]]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'standard_write_results_event':
            self.results = values['standard_write_results_event']
            self.fill_table_values(self.results[2], window)

        if event == 'standard_configtable_back_button':
            window['standard_configtable_panel'].update(visible=False)
            window['standard_result_panel'].update(visible=True)

    def fill_table_values(self, values, window):
        window['standard_configtable_configuration_text'].update(values[0][1])
        window['standard_configtable_configuration_text'].set_size((len(values[0][1]), 1))

        window['standard_configtable_model_text'].update(values[1][1])
        window['standard_configtable_model_text'].set_size((len(values[1][1]), 1))

        window['standard_configtable_variable_text'].update('\n'.join(values[2][1]))
        window['standard_configtable_variable_text'].set_size(
            (len(max(values[2][1], key=len)) + 2, len(values[2][1])))

        window['standard_configtable_repetition_text'].update(values[3][1])
        window['standard_configtable_repetition_text'].set_size((10, 1))

        window['standard_configtable_tick_text'].update(values[4][1])
        window['standard_configtable_tick_text'].set_size((10, 1))

        window['standard_configtable_reporter_text'].update('\n'.join(values[5][1]))
        window['standard_configtable_reporter_text'].set_size(
            (len(max(values[5][1], key=len)) + 2, len(values[5][1])))

        window['standard_configtable_setup_text'].update('\n'.join(values[6][1]))
        window['standard_configtable_setup_text'].set_size(
            (len(max(values[6][1], key=len)) + 2, len(values[6][1])))

        window['standard_configtable_process_text'].update(values[7][1])
        window['standard_configtable_process_text'].set_size((10, 1))



import pyNetLogo as pnl
import math
from pyNetLogo.core import NetLogoException


def run_standard(child_conn, values, repetitions, netlogo_version, netlogo_home):
    outcomes = {}
    ticks = int(values['standard_tick_input'])
    steps = 100 / repetitions
    progress_bar = 1

    reporters = split_user_input(values['standard_reporter_input'])
    for reporter in reporters:
        outcomes[reporter] = []

    try:
        netlogo = pnl.NetLogoLink(gui=False, netlogo_version=netlogo_version, netlogo_home=netlogo_home)
    except (TypeError, KeyError):
        child_conn.send(get_failure_message('Error in input: Invalid NetLogo version or path. Please make sure the '
                                            'values in gui_user.py are correct (this requires a restart of the GUI).'))
        return

    model = values['standard_model_input']
    try:
        netlogo.load_model(model)
    except (FileNotFoundError, NetLogoException):
        child_conn.send(get_failure_message('Error in input: Invalid model file path. Please make sure the file path is'
                                            ' correct and points to a valid .nlogo file.'))
        return

    try:
        variable_dict = get_variable_dictionary(values['standard_value_input'])
    except (ValueError, IndexError):
        child_conn.send(get_failure_message('Error in input: Invalid parameter input. Please '
                                            'make sure the input uses the correct format.'))
        return

    for key in variable_dict.keys():
        if key == 'random-seed':
            netlogo.command(f'random-seed {variable_dict[key]}')
        else:
            try:
                netlogo.command(f'set {key} {variable_dict[key]}')
            except NetLogoException:
                child_conn.send(get_failure_message('Error in input: Invalid parameter input. Please '
                                                    'make sure all variable names and their values are correct.'))
                netlogo.kill_workspace()
                return

    for run in range(repetitions):
        setup_commands = split_user_input(values['standard_setup_input'])
        for command in setup_commands:
            try:
                netlogo.command(command)
            except NetLogoException:
                child_conn.send(get_failure_message('Error in input: Invalid setup commands. Please '
                                                    'make sure they\'re all valid NetLogo commands'))
                netlogo.kill_workspace()
                return

        try:
            results = netlogo.repeat_report(reporters, ticks)
        except NetLogoException:
            child_conn.send(get_failure_message('Error in input: Invalid reporter commands. Please '
                                                'make sure all reporters are valid NetLogo commands'))
            netlogo.kill_workspace()
            return

        for reporter in outcomes.keys():
            outcomes[reporter].append(list(results[reporter]))

        if run * steps > progress_bar:
            progress_bar = math.ceil(progress_bar + steps)
            if steps < 1:
                child_conn.send({'Progress': 1, 'Results': []})
            else:
                child_conn.send({'Progress': steps, 'Results': []})

    netlogo.kill_workspace()

    results = variable_dict, outcomes
    child_conn.send({'Progress': steps, 'Results': results})


def split_user_input(input):
    rows = input.split('\n')
    rows = list(filter(('').__ne__, rows))
    mapped_rows = list(map(lambda x: x.strip(), rows))
    return mapped_rows



def get_variable_dictionary(var_values):
    rows = var_values.split('\n')
    rows = list(filter(('').__ne__, rows))
    stripped_rows = list(map(lambda x: x.strip().split(' '), rows))
    return {x[0]: int(x[1]) for x in stripped_rows}


def get_failure_message(msg):
    return {'Progress': 'Failure', 'Results': msg}



from setuptools import setup, find_packages
setup(
  name = 'NetLogoDOE',
  packages = find_packages(),
  include_package_data=True,
  version = '0.1.19',
  license='MIT',
  description = 'NetLogoDOE provides a GUI that allows for easy design, execution and analysis of NetLogo experiments',
  author = 'Robin Faber',
  author_email = 'r.j.faber@student.tudelft.nl',
  url = 'https://github.com/robinfaber97/NetLogoDOE',
  download_url = 'https://github.com/robinfaber97/NetLogoDOE/archive/refs/tags/0.1.19.tar.gz',
  keywords = ['NetLogo', 'Design of experiments', 'Model design', 'Agent-based simulation', 'Data visualisation'],
  install_requires=[
          'PySimpleGUI',
          'plotly',
          'statsmodels',
          'pandas',
          'pyNetLogo',
          'numpy',
          'pyDOE2',
          'SALib',
          'matplotlib',
          'seaborn',
          'scipy',
          'future',
          'jpype1',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
)



import multiprocessing
import traceback
import pyNetLogo as pnl


class Process(multiprocessing.Process):
    def __init__(self, *args, **kwargs):
        multiprocessing.Process.__init__(self, *args, **kwargs)
        self._pconn, self._cconn = multiprocessing.Pipe()
        self._exception = None

    def run(self):
        try:
            multiprocessing.Process.run(self)
            self._cconn.send(None)
        except Exception as e:
            tb = traceback.format_exc()
            self._cconn.send((e, tb))

    @property
    def exception(self):
        if self._pconn.poll():
            self._exception = self._pconn.recv()
        return self._exception


class Runner:

    def __init__(self, function, args):
        self.parent_conn, self.child_conn = multiprocessing.Pipe()
        self.child_process = Process(target=function, args=((self.child_conn,) + args))

    def run(self):
        self.child_process.start()

    def get_result(self):
        if self.parent_conn.poll(1):
            child_return = self.parent_conn.recv()
            return child_return
        return None


import PySimpleGUI as sg


def show_error_window(text):
    layout = [[sg.Text(text)],
              [sg.CloseButton('Ok')]]
    error_window = sg.Window('Error', layout=layout, element_justification='c', finalize=True)

    error_window.read()


def show_help_window(text, location=(600, 100)):
    layout = [[sg.Text(text)],
              [sg.CloseButton('Ok')]]
    error_window = sg.Window('Help', layout=layout, element_justification='c', finalize=True,
                             location=(location[0]+100, location[1]+100))

    error_window.read()



import PySimpleGUI as sg
import plotly.graph_objects as go
from NetLogoDOE.src.util.data_processing.merge_standard_data import merge_data
from NetLogoDOE.src.gui.custom_components import title, metric3_radio_buttons, question_mark_button
from NetLogoDOE.src.gui.custom_windows import show_help_window
from NetLogoDOE.src.gui.help_dictionary import help_text


class HistogramScreen:

    def __init__(self):
        self.layout = [[title('Histogram')],
                       [sg.Text('Graph Title: '), sg.Input('', key='histogram_title_input')],
                       [question_mark_button('histogram_reporter_help_button'), sg.Text('Reporters to plot:')],
                       [sg.Multiline('', key='histogram_reporter_input')],
                       [sg.Text('Only input a single reporter on each line')],
                       [question_mark_button('histogram_3metric_help_button'), sg.Text('Metric:')],
                       metric3_radio_buttons('histogram'),
                       [sg.Button('Generate', key='histogram_generate_button')],
                       [sg.Button('Back', key='histogram_back_button')]]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'standard_write_results_event':
            self.results = values['standard_write_results_event']
        if event == 'histogram_generate_button':
            valid, error_message = self.validate_user_input(values)
            if not valid:
                window.write_event_value('show_error_window', error_message)
                return
            self.generate_histogram(values, window)
        if event == 'histogram_back_button':
            window['histogram_panel'].update(visible=False)
            window['standard_result_panel'].update(visible=True)

        # Help events
        if event == 'histogram_reporter_help_button':
            show_help_window(help_text['standard_plot_reporters'], location=window.CurrentLocation())
        if event == 'histogram_3metric_help_button':
            show_help_window(help_text['3_metric'], location=window.CurrentLocation())

    def generate_histogram(self, values, window):
        reporters = self.format_reporters(values)
        metric = self.get_selected_radio_button(values)
        merged_data = merge_data(self.results[1], metric)
        try:
            [merged_data[reporter] for reporter in reporters]
        except KeyError:
            window.write_event_value('show_error_window', 'Error in input: Invalid reporter name.\nPlease make '
                                                          'sure all reporters are valid with the current run '
                                                          'configuration.')
            return
        fig = go.Figure()
        for reporter in reporters:
            fig.add_trace(go.Histogram(x=merged_data[reporter], name=reporter))

        fig.update_layout(title_text=values['histogram_title_input'],
                          xaxis_title_text='Value',
                          yaxis_title_text='Count'
                          )
        fig.show()

    def get_selected_radio_button(self, values):
        for i in range(3):
            if values[f'histogram_metric_{i + 1}']:
                return i + 1
        return 1

    def format_reporters(self, values):
        reporters = list(filter(('').__ne__, values['histogram_reporter_input'].split('\n')))
        reporters = list(map(lambda x: x.strip(), reporters))
        return reporters

    def validate_user_input(self, values):
        if values['histogram_reporter_input'] == '\n':
            return False, 'Error in input: One or more fields are empty. ' \
                          'Please make sure all options are filled out.'

        return True, ''



import PySimpleGUI as sg
import plotly.graph_objects as go
import plotly.express as px
from plotly.subplots import make_subplots
import statsmodels.api as sm
import re
import math
from NetLogoDOE.src.util.data_processing.merge_experiment_data import merge_data
from NetLogoDOE.src.gui.custom_components import title, metric4_radio_buttons, question_mark_button
from NetLogoDOE.src.gui.custom_windows import show_help_window
from NetLogoDOE.src.gui.help_dictionary import help_text


class ScatterplotScreen:

    def __init__(self):
        self.layout = [[title('Scatterplot')],
                       [question_mark_button('scatterplot_variable_help_button'), sg.Text('Variable combinations to plot:')],
                       [sg.Multiline('"sheep-reproduce" "count sheep"', key='scatterplot_variable_input')],
                       [sg.Text('Explain the format: "x-variable" "y-variable"')],
                       [question_mark_button('scatterplot_4metric_help_button'), sg.Text('Metric:')],
                       metric4_radio_buttons('scatter'),
                       [sg.Button('Generate', key='scatterplot_generate_button')],
                       [sg.Button('Back', key='scatterplot_back_button')]]
        self.results = None

    def check_events(self, event, values, window):
        if event == 'experiment_write_results_event':
            self.results = values['experiment_write_results_event']
        if event == 'scatterplot_generate_button':
            valid, error_message = self.validate_user_input(values)
            if not valid:
                window.write_event_value('show_error_window', error_message)
                return
            self.generate_scatter_plot(values, window)
        if event == 'scatterplot_back_button':
            window['scatterplot_panel'].update(visible=False)
            window['experiment_result_panel'].update(visible=True)

        # Help events
        if event == 'scatterplot_4metric_help_button':
            show_help_window(help_text['4_metric'], location=window.CurrentLocation())
        if event == 'scatterplot_variable_help_button':
            show_help_window(help_text['scatterplot_variables'], location=window.CurrentLocation())

    def generate_scatter_plot(self, values, window):
        exp = self.results[0]
        outcomes = self.results[1]

        repetitions = len(self.results[1][list(self.results[1].keys())[0]][0])
        metric = self.get_selected_radio_button(values)
        merged_data = merge_data(exp, outcomes, repetitions, metric)
        axis_variables = self.format_scatter_axis(values['scatterplot_variable_input'])
        try:
            [merged_data[var] for variables in axis_variables for var in variables]
        except KeyError:
            window.write_event_value('show_error_window', 'Error in input: Invalid variable name.\nPlease make '
                                                          'sure all x and y variables are either a model parameter or '
                                                          'NetLogo reporter used in the experiment configuration.')
            return

        rows, cols = self.get_subplots_format(len(axis_variables))
        titles = [f'Effect of "{variables[0]}" on "{variables[1]}"'
                  for variables in axis_variables]

        fig = make_subplots(rows=rows, cols=cols, subplot_titles=titles)
        colors = px.colors.qualitative.Plotly
        for i, variables in enumerate(axis_variables):
            x = merged_data[variables[0]]
            y = merged_data[variables[1]]
            row = math.floor(i / cols) + 1
            col = (i % cols) + 1

            fig.append_trace(go.Scatter(x=x, y=y, mode='markers', showlegend=False,
                                        line=dict(color=colors[i % 9])), row=row, col=col)

            fit_results = sm.OLS(y, sm.add_constant(x), missing="drop").fit()
            res = fit_results.predict()
            fig.append_trace(go.Scatter(x=x, y=res, mode='lines', showlegend=False,
                                        line=dict(color=colors[i % 9])), row=row, col=col)
            fig.update_xaxes(title_text=variables[0], row=row, col=col)
            fig.update_yaxes(title_text=variables[1], row=row, col=col)

        fig.show()

    def get_selected_radio_button(self, values):
        for i in range(4):
            if values[f'scatter_metric_{i + 1}']:
                return i + 1
        return 1

    def format_scatter_axis(self, variables):
        rows = list(filter(('').__ne__, variables.split('\n')))
        mapped_rows = list(map(lambda x: re.findall('"([^"]*)"', x), rows))
        mapped_rows = list(map(lambda x: (x[0], x[1]), mapped_rows))
        return mapped_rows

    def get_subplots_format(self, subplots):
        if subplots == 1:
            return 1, 1
        elif subplots == 2:
            return 1, 2
        elif subplots == 3:
            return 1, 3
        elif subplots == 4:
            return 2, 2
        else:
            rows = math.floor(subplots / 3)
            if subplots % 3 == 0:
                return rows, 3
            else:
                return rows + 1, 3

    def validate_user_input(self, values):
        if values['scatterplot_variable_input'] == '\n':
            return False, 'Error in input: One or more fields are empty. ' \
                          'Please make sure all options are filled out.'

        try:
            self.format_scatter_axis(values['scatterplot_variable_input'])
        except IndexError:
            return False, 'Error in input: Incorrect format for the variables to plot.\n' \
                          'Please make sure the format is correct, specifically the double quotes.'

        return True, ''



import PySimpleGUI as sg

from NetLogoDOE.src.gui.custom_windows import show_error_window

from NetLogoDOE.src.gui.navigation.MainScreen import MainScreen
from NetLogoDOE.src.gui.navigation.ExperimentScreen import ExperimentScreen
from NetLogoDOE.src.gui.navigation.StandardScreen import StandardScreen
from NetLogoDOE.src.gui.navigation.ImportScreen import ImportScreen
from NetLogoDOE.src.gui.navigation.RunScreen import RunScreen
from NetLogoDOE.src.gui.navigation.ExperimentResultsScreen import ExperimentResultsScreen
from NetLogoDOE.src.gui.navigation.StandardResultsScreen import StandardResultsScreen
from NetLogoDOE.src.gui.plots.standard.StandardConfigTableScreen import StandardConfigTableScreen
from NetLogoDOE.src.gui.plots.experiment.ExperimentConfigTableScreen import ExperimentConfigTableScreen
from NetLogoDOE.src.gui.plots.experiment.ParallelCoordinatesScreen import ParallelCoordinatesScreen
from NetLogoDOE.src.gui.plots.experiment.ScatterplotScreen import ScatterplotScreen
from NetLogoDOE.src.gui.plots.experiment.HeatmapScreen import HeatmapScreen
from NetLogoDOE.src.gui.plots.standard.TimeSeriesplotScreen import TimeSeriesplotScreen
from NetLogoDOE.src.gui.plots.standard.BoxplotScreen import BoxplotScreen
from NetLogoDOE.src.gui.plots.standard.ViolinplotScreen import ViolinplotScreen
from NetLogoDOE.src.gui.plots.standard.HistogramScreen import HistogramScreen
from NetLogoDOE.src.gui.plots.standard.DistributionplotScreen import DistributionplotScreen


class Gui:
    def __init__(self, netlogo_version=None, netlogo_home=None):
        sg.theme('DefaultNoMoreNagging')
        self.main_screen = MainScreen()
        self.experiment_screen = ExperimentScreen()
        self.standard_screen = StandardScreen()
        self.import_screen = ImportScreen()
        self.run_screen = RunScreen(netlogo_version, netlogo_home)
        self.experiment_result_screen = ExperimentResultsScreen()
        self.standard_result_screen = StandardResultsScreen()

        # Plot screens
        self.standard_configtable_screen = StandardConfigTableScreen()
        self.experiment_configtable_screen = ExperimentConfigTableScreen()
        self.parallel_coordinates_screen = ParallelCoordinatesScreen()
        self.scatterplot_screen = ScatterplotScreen()
        self.heatmap_screen = HeatmapScreen()
        self.timeseries_screen = TimeSeriesplotScreen()
        self.boxplot_screen = BoxplotScreen()
        self.violinplot_screen = ViolinplotScreen()
        self.histogram_screen = HistogramScreen()
        self.distplot_screen = DistributionplotScreen()

        window = self.create_gui()
        self.run(window)

    def create_gui(self):
        layout_main = self.main_screen.layout
        layout_experiment = self.experiment_screen.layout
        layout_standard = self.standard_screen.layout
        layout_import = self.import_screen.layout
        layout_run = self.run_screen.layout
        layout_experiment_result = self.experiment_result_screen.layout
        layout_standard_result = self.standard_result_screen.layout

        # Plot layouts
        layout_standard_configtable = self.standard_configtable_screen.layout
        layout_experiment_configtable = self.experiment_configtable_screen.layout
        layout_parallel_coordinates = self.parallel_coordinates_screen.layout
        layout_scatterplot = self.scatterplot_screen.layout
        layout_heatmap = self.heatmap_screen.layout
        layout_timeseries = self.timeseries_screen.layout
        layout_boxplot = self.boxplot_screen.layout
        layout_violinplot = self.violinplot_screen.layout
        layout_histogram = self.histogram_screen.layout
        layout_distplot = self.distplot_screen.layout

        layout_gui = [[sg.Column(layout_main, key='main_panel', element_justification='center'),
                       sg.Column(layout_experiment, key="experiment_panel", visible=False),
                       sg.Column(layout_standard, key="standard_panel", visible=False),
                       sg.Column(layout_import, key="import_panel", visible=False, element_justification='center'),
                       sg.Column(layout_run, key='run_panel', visible=False),
                       sg.Column(layout_experiment_result, key="experiment_result_panel", visible=False, element_justification='center'),
                       sg.Column(layout_standard_result, key="standard_result_panel", visible=False, element_justification='center'),
                       sg.Column(layout_standard_configtable, key="standard_configtable_panel", visible=False),
                       sg.Column(layout_experiment_configtable, key="experiment_configtable_panel", visible=False),
                       sg.Column(layout_parallel_coordinates, key="parcoords_panel", visible=False),
                       sg.Column(layout_scatterplot, key="scatterplot_panel", visible=False),
                       sg.Column(layout_heatmap, key="heatmap_panel", visible=False),
                       sg.Column(layout_timeseries, key="timeseries_panel", visible=False),
                       sg.Column(layout_boxplot, key="boxplot_panel", visible=False),
                       sg.Column(layout_violinplot, key="violinplot_panel", visible=False),
                       sg.Column(layout_histogram, key="histogram_panel", visible=False),
                       sg.Column(layout_distplot, key="distplot_panel", visible=False),
                       ]]

        # Create the window
        return sg.Window("NetLogoDOE", layout_gui, resizable=False, location=(600, 100))

    def run(self, window):
        while True:
            event, values = window.read()
            if event == 'main_close_button' or event == sg.WIN_CLOSED:
                break

            if event == 'show_error_window':
                show_error_window(values['show_error_window'])

            self.main_screen.check_events(event, values, window)
            self.experiment_screen.check_events(event, values, window)
            self.standard_screen.check_events(event, values, window)
            self.import_screen.check_events(event, values, window)
            self.run_screen.check_events(event, values, window)
            self.experiment_result_screen.check_events(event, values, window)
            self.standard_result_screen.check_events(event, values, window)

            self.standard_configtable_screen.check_events(event, values, window)
            self.experiment_configtable_screen.check_events(event, values, window)
            self.parallel_coordinates_screen.check_events(event, values, window)
            self.scatterplot_screen.check_events(event, values, window)
            self.heatmap_screen.check_events(event, values, window)
            self.timeseries_screen.check_events(event, values, window)
            self.boxplot_screen.check_events(event, values, window)
            self.violinplot_screen.check_events(event, values, window)
            self.histogram_screen.check_events(event, values, window)
            self.distplot_screen.check_events(event, values, window)
            
        window.close()
